import 'dart:async';
import 'dart:convert';

import 'package:flulm_auth/src/model/result_data.dart';
import 'package:flulm_auth/src/utils/constant.dart';
import 'package:flulm_auth/src/utils/http_manager.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:webview_flutter/webview_flutter.dart';

///**
// * @author dashu
// * @date 6/15/20
// * describe:
// * 支付H5
// */
class PayPage extends StatefulWidget {
  final String signData;
  final int isTest;
  final String type;
  final String language;
  final bool supportScan;

  PayPage(
      {this.type,
      this.signData,
      this.isTest,
      this.language,
      this.supportScan,
      Key key})
      : super(key: key);

  @override
  _PayPageState createState() => _PayPageState();
}

class _PayPageState extends State<PayPage> {
  String webUrl = "";

//  final Completer<WebViewController> _webViewController =
//      Completer<WebViewController>();
  WebViewController _webViewController;

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<ResultData>(
        future: getLMPayH5Url(widget.type, widget.signData, widget.isTest),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.done) {
            if (snapshot.data == null || !snapshot.data.isSuccess()) {
              return SafeArea(
                child: Scaffold(
                  backgroundColor: Theme.of(context).primaryColor,
                  body: Center(
                    child: Column(
                      mainAxisSize: MainAxisSize.min,
                      children: <Widget>[
                        Text(
                          "Error: ${snapshot.data?.msg ?? '获取H5地址失败'}",
                          style: TextStyle(color: Colors.red),
                        ),
                        SizedBox(
                          height: 30.0,
                        ),
                        RaisedButton(
                          onPressed: () {
                            goBackError(
                                snapshot.data.code, snapshot.data.msg, '');
                          },
                          child: Padding(
                            padding: const EdgeInsets.all(8.0),
                            child: Text(
                              '返回',
                              style: TextStyle(
                                fontSize: 30.0,
                                color: Colors.white,
                              ),
                            ),
                          ),
                          color: Theme.of(context).primaryColor,
                        )
                      ],
                    ),
                  ),
                ),
              );
            } else {
              return WillPopScope(
                onWillPop: () async {
                  _loadBackPressed();
                  return false;
                },
                child: SafeArea(
                  child: Scaffold(
                    body: Container(
                      width: Constant.getResetWidth(context),
                      child: WebView(
                        userAgent: Constant.userAgentSuffix + widget.language,
                        //链接编码
                        initialUrl: Uri.encodeFull(webUrl),
                        javascriptMode: JavascriptMode.unrestricted,
                        onWebViewCreated: (WebViewController controller) {
                          debugPrint("onWebViewCreated===");
                          _webViewController = controller;
                        },
                        javascriptChannels: <JavascriptChannel>[
                          Constant.alertJavascriptChannel(_dispatchJs),
                        ].toSet(),
                        navigationDelegate: (NavigationRequest request) {
                          if (!request.url.startsWith('http')) {
                            return NavigationDecision.prevent;
                          }
                          debugPrint('allowing navigation to $request');
                          return NavigationDecision.navigate;
                        },
                        onPageFinished: (String url) {
                          debugPrint("onPageFinished=" + url);
                        },
                        debuggingEnabled: Constant.isDebug,
                      ),
                    ),
                  ),
                ),
              );
            }
          } else {
            return Container(
              child: Center(
                child: CircularProgressIndicator(
                  backgroundColor: Colors.white,
                ),
              ),
              color: Colors.blue,
            );
          }
        });
  }

//  /**
//   * 返回键 让前端处理
//   */
  void _loadBackPressed() {
    _webViewController?.evaluateJavascript('onBackPressed()')?.then((result) {
      // You can handle JS result here.
      debugPrint("_loadBackPressed=" + result);
      if (result == null) {
        goBackError(-1, "用户已取消", "");
      }
    });
  }

  Future<ResultData> getLMPayH5Url(String type, String data, int isTest) async {
    String host;
    switch (isTest) {
      case 1:
        host = "https://testsdkapi.lmpayplc.com";
        break;
      case 2:
        host = "http://sdkapi1.lmpayplc.com";
        break;
      case 0:
      default:
        host = "https://sdkapi.lmpayplc.com";
    }

    Map<String, String> jsonMap = Map();
    jsonMap["app_type"] = type;
    jsonMap["data"] = data;
    jsonMap["from"] = "flutter-" + defaultTargetPlatform.toString();

    Map<String, dynamic> map = Map();
    map["post_data"] = jsonEncode(jsonMap);
    ResultData resultData =
        await HttpManager.getInstance(host).post("?ct=index&ac=init_sdk", map);
    if (resultData.isSuccess()) {
      webUrl = resultData.data["h5_url"];
      if (webUrl?.isNotEmpty == true) {
        webUrl = webUrl + "&noscan=" + (widget.supportScan ? 0 : 1).toString();
      }
    }

    print("getLMPayH5Url:" + webUrl);
    return resultData;
  }

  void goBackError(int code, String msg, String data) {
    Map<String, dynamic> map = Map();
    map['errCode'] = code;
    map['errStr'] = msg;
    map['resultData'] = data;
    Navigator.pop(context, map);
  }

  //分发数据
  void _dispatchJs(String method, Map<String, dynamic> jsonData) {
    switch (method) {
      case "onPayResultCall":
        {
          onPayResultCall(jsonData['code'], jsonData['message'],
              jsonData['result_data'].toString());
          break;
        }

      case "closePage":
        {
          closePage(jsonData['code']);
          break;
        }
      default:
        {
          debugPrint("没有匹配");
        }
        break;
    }
  }

  void onPayResultCall(int code, String message, String respJsonData) {
    goBackError(code, message, respJsonData);
  }

  void closePage(int status) {
    debugPrint("closePage == status=" + status.toString());
    if (status == 0) {
      Navigator.pop(context);
    } else {
      showExitDialog();
    }
  }

//  @JavascriptInterface
//  public void getLoginCodeSuccess(String code) {
//    goBack2ThirdApp(0, "授权成功", code);
//  }
//
//  @JavascriptInterface
//  public void onAuthResultCall(int code, String message) {
//    goBack2ThirdApp(code, message, "");
//  }

  void showExitDialog() {
    String title = "Are you sure to exit?";
    String btn = "Exit";
    if (widget.language == "zh-cn") {
      title = "是否确定退出？";
      btn = "退出";
    } else if (widget.language == "km") {
      title = "តើអ្នកប្រាកដជាចាកចេញឬ?";
      btn = "បោះបង់ការសិក្សា";
    }
    AlertDialog(
      content: Text(title),
      actions: <Widget>[
//        FlatButton(
//          child: Text("取消"),
//          onPressed: () => Navigator.of(context).pop(), //关闭对话框
//        ),
        FlatButton(
          child: Text(btn),
          onPressed: () {
            Navigator.pop(context); //关闭对话框
          },
        ),
      ],
    );
  }
}
