import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flulm_auth/src/model/result_data.dart';

import 'logs_interceptors.dart';

class HttpManager {
  Dio _dio;

  static HttpManager getInstance(String baseUrl) {
    return _instance._baseUrl(baseUrl);
  }

  factory HttpManager() => _instance;

  static HttpManager _instance = HttpManager._internal();

  HttpManager._internal({String baseUrl}) {
    if (null == _dio) {
      _dio = Dio();
      _dio.options.baseUrl = baseUrl;
      _dio.options.connectTimeout = 5000; //5s
      _dio.options.receiveTimeout = 5000;
      _dio.options.headers = {
        HttpHeaders.userAgentHeader: 'dio',
//        'common-header': 'flutter_lema'
      };
      _dio.interceptors.add(LogsInterceptors());
    }
  }

  //用于指定特定域名，比如cdn和kline首次的http请求
  HttpManager _baseUrl(String baseUrl) {
    if (_dio != null && baseUrl != _dio.options.baseUrl) {
      _dio.options.baseUrl = baseUrl;
    }
    return this;
  }

  ///通用的GET请求
  get(url, params, {noTip = false}) async {
    Response response;
    try {
      response = await _dio.get(url, queryParameters: params);
    } on DioError catch (e) {
      return resultError(e);
    }

//    if (response.data is DioError) {
//      return resultError(response.data['code']);
//    }
    return ResultData(
        response.data['code'], response.data['msg'], response.data['data']);
  }

  ///通用的POST请求
  post(String url, Map<String, dynamic> params) async {
    Response response;

//    print("params-> $params");

    try {
      response = await _dio.request(url,
          data: params,
          options: Options(
              contentType:
                  ContentType.parse("application/x-www-form-urlencoded")
                      .toString(),
              method: "post"));
    } on DioError catch (e) {
      return resultError(e);
    }

//    if (response.data is DioError) {
//      return resultError(response.data['code']);
//    }
    return ResultData(
        response.data['code'], response.data['msg'], response.data['data']);
  }

  ResultData resultError(DioError e) {
    print("resultError=" + e.message);
    return ResultData(-1, e.message, null);
  }
}
